@@ -20,14 +20,14 @@
 
 #include "frame_glom.h"
 #include "application.h"
-#include "appstate.h"
+#include <glom/libglom/appstate.h>
 #include "mode_design/users/dialog_groups_list.h"
 #include "dialog_database_preferences.h"
 #include "reports/dialog_layout_report.h"
-#include "utils.h"
-#include "data_structure/glomconversions.h"
-#include "data_structure/layout/report_parts/layoutitem_summary.h"
-#include "data_structure/layout/report_parts/layoutitem_fieldsummary.h"
+#include <glom/libglom/utils.h>
+#include <glom/libglom/data_structure/glomconversions.h>
+#include <glom/libglom/data_structure/layout/report_parts/layoutitem_summary.h>
+#include <glom/libglom/data_structure/layout/report_parts/layoutitem_fieldsummary.h>
 #include "relationships_overview/dialog_relationships_overview.h"
 #include "filechooser_export.h"
 #include <sstream> //For stringstream.
@@ -678,6 +678,10 @@ void Frame_Glom::on_menu_Mode_Data()
 
 void Frame_Glom::on_menu_Mode_Find()
 {
+  //This can take quite a long time, flicking between 1 or 2 intermediate screens. 
+  //It shouldn't, but until we fix that, let's show the busy cursor while it's working:
+  Bakery::BusyCursor busy_cursor(get_app_window());
+
   const bool previously_in_data_mode = (m_Mode == MODE_Data);
 
   const Notebook_Data::dataview list_or_details = m_Notebook_Data.get_current_view();
@@ -830,17 +834,23 @@ void Frame_Glom::on_notebook_find_criteria(const Glib::ustring& where_clause)
   App_Glom* pApp = dynamic_cast<App_Glom*>(get_app_window());
   if(pApp)
   {
-    pApp->set_mode_data();
+    bool records_found = false;
 
-    //std::cout << "Frame_Glom::on_notebook_find_criteria: where_clause=" << where_clause << std::endl;
-    FoundSet found_set;
-    found_set.m_table_name = m_table_name;
-    found_set.m_where_clause = where_clause;
-    const bool records_found = m_Notebook_Data.init_db_details(found_set);
+    { //Extra scope, to control the lifetime of the busy cursor. 
+      Bakery::BusyCursor busy_cursor(pApp);
 
-    //std::cout << "Frame_Glom::on_notebook_find_criteria(): BEFORE  m_Notebook_Data.select_page_for_find_results()" << std::endl;
-    m_Notebook_Data.select_page_for_find_results();
-    //std::cout << "Frame_Glom::on_notebook_find_criteria(): AFTER  m_Notebook_Data.select_page_for_find_results()" << std::endl;
+      pApp->set_mode_data();
+
+      //std::cout << "Frame_Glom::on_notebook_find_criteria: where_clause=" << where_clause << std::endl;
+      FoundSet found_set;
+      found_set.m_table_name = m_table_name;
+      found_set.m_where_clause = where_clause;
+      records_found = m_Notebook_Data.init_db_details(found_set);
+
+      //std::cout << "Frame_Glom::on_notebook_find_criteria(): BEFORE  m_Notebook_Data.select_page_for_find_results()" << std::endl;
+      m_Notebook_Data.select_page_for_find_results();
+      //std::cout << "Frame_Glom::on_notebook_find_criteria(): AFTER  m_Notebook_Data.select_page_for_find_results()" << std::endl;
+    }
 
     if(!records_found)
     {
