@@ -19,11 +19,12 @@
  */
 
 #include "box_data_details.h"
-#include "../data_structure/field.h"
-#include "../data_structure/relationship.h"
-#include "../data_structure/glomconversions.h"
+#include <glom/libglom/data_structure/field.h>
+#include <glom/libglom/data_structure/relationship.h>
+#include <glom/libglom/data_structure/glomconversions.h>
 #include "dialog_layout_details.h"
-#include "../utils.h"
+#include <glom/libglom/utils.h>
+#include "../xsl_utils.h"
 #include <bakery/App/App_Gtk.h> //For util_bold_message().
 #include "../python_embed/glom_python.h"
 #include <sstream> //For stringstream
@@ -117,7 +118,9 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
   m_FlowTable.signal_layout_changed().connect( sigc::mem_fun(*this, &Box_Data_Details::on_flowtable_layout_changed) );
 
   m_FlowTable.signal_requested_related_details().connect( sigc::mem_fun(*this, &Box_Data_Details::on_flowtable_requested_related_details) );
-    
+
+  m_FlowTable.signal_script_button_clicked().connect( sigc::mem_fun(*this, &Box_Data_Details::on_flowtable_script_button_clicked) );
+
   m_ignore_signals = false;
 }
 
@@ -131,29 +134,32 @@ Gnome::Gda::Value Box_Data_Details::get_primary_key_value() const
   return m_primary_key_value;
 }
 
-bool Box_Data_Details::init_db_details(const Glib::ustring& strTableName, const Gnome::Gda::Value& primary_key_value)
+bool Box_Data_Details::init_db_details(const FoundSet& found_set, const Gnome::Gda::Value& primary_key_value)
 {
+  //std::cout << "Box_Data_Details::init_db_details(): primary_key_value.to_string()=" << primary_key_value.to_string() << std::endl;
+
   m_primary_key_value = primary_key_value;
 
-  get_field_primary_key_for_table(strTableName, m_field_primary_key);
+  m_field_primary_key = get_field_primary_key_for_table(found_set.m_table_name);
 
-  return Box_Data::init_db_details(strTableName); //Calls create_layout(), then fill_from_database()
+  return Box_Data::init_db_details(found_set); //Calls create_layout(), then fill_from_database()
 }
 
-bool Box_Data_Details::refresh_data_from_database(const Gnome::Gda::Value& primary_key_value)
+bool Box_Data_Details::refresh_data_from_database_with_primary_key(const Gnome::Gda::Value& primary_key_value)
 {
+  //std::cout << "refresh_data_from_database_with_primary_key(): primary_key_value.to_string()=" << primary_key_value.to_string() << std::endl;
   m_primary_key_value = primary_key_value;
   return fill_from_database();
 }
 
 bool Box_Data_Details::refresh_data_from_database_blank()
 {
-  return refresh_data_from_database( Gnome::Gda::Value() );
+  return refresh_data_from_database_with_primary_key( Gnome::Gda::Value() );
 }
 
 void Box_Data_Details::create_layout()
 {
-  Bakery::BusyCursor(*get_app_window());
+  Bakery::BusyCursor busy_cursor(get_app_window());
 
   Box_Data::create_layout(); //Fills m_TableFields.
 
@@ -163,7 +169,7 @@ void Box_Data_Details::create_layout()
   Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
   if(document)
   {
-    m_FlowTable.set_table(m_strTableName); //This allows portals to get full Relationship information
+    m_FlowTable.set_table(m_table_name); //This allows portals to get full Relationship information
 
     //This map of layout groups will also contain the field information from the database:
     Document_Glom::type_mapLayoutGroupSequence layout_groups = get_data_layout_groups(m_layout_name);
@@ -178,20 +184,22 @@ void Box_Data_Details::create_layout()
 
 bool Box_Data_Details::fill_from_database()
 {
+  //std::cout << "Box_Data_Details::fill_from_database(): m_primary_key_value.to_string()=" << m_primary_key_value.to_string() << std::endl;
+
   bool bResult = false;
 
-  Bakery::BusyCursor(*get_app_window());
+  Bakery::BusyCursor busy_cursor(get_app_window());
 
   const bool primary_key_is_empty = GlomConversions::value_is_empty(m_primary_key_value);
   if(!primary_key_is_empty)
-    get_document()->set_layout_record_viewed(m_strTableName, m_layout_name, m_primary_key_value);
-    
+    get_document()->set_layout_record_viewed(m_table_name, m_layout_name, m_primary_key_value);
+
   try
   {
 
     //TODO: This should keep the connection open, so we don't need to 
     //reconnect many times..
-    sharedptr<SharedConnection> sharedconnection = connect_to_server();
+    sharedptr<SharedConnection> sharedconnection = connect_to_server(get_app_window());
 
 
     bResult = Box_Data::fill_from_database();
@@ -202,7 +210,7 @@ bool Box_Data_Details::fill_from_database()
     if(!fieldsToGet.empty())
     {
       //Do not try to show the data if the user may not view it:
-      Privileges table_privs = get_current_privs(m_strTableName);
+      Privileges table_privs = get_current_privs(m_table_name);
 
       //Enable/Disable record creation and deletion:
       m_Button_New.set_sensitive(table_privs.m_create);
@@ -211,18 +219,18 @@ bool Box_Data_Details::fill_from_database()
       if(table_privs.m_view)
       {
         //Add extra possibly-non-visible columns that we need:
-        sharedptr<LayoutItem_Field> layout_item(new LayoutItem_Field);
-        layout_item->m_field = m_field_primary_key;
+        sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+        layout_item->set_full_field_details(m_field_primary_key);
         fieldsToGet.push_back(layout_item);
 
-        //g_warning("primary_key name = %s", m_field_primary_key.get_name().c_str());
+        //g_warning("primary_key name = %s", m_field_primary_key->get_name().c_str());
         const int index_primary_key = fieldsToGet.size() - 1;
 
-        const Glib::ustring query = build_sql_select(m_strTableName, fieldsToGet, m_field_primary_key, m_primary_key_value);
+        const Glib::ustring query = GlomUtils::build_sql_select_with_key(m_table_name, fieldsToGet, m_field_primary_key, m_primary_key_value);
         Glib::RefPtr<Gnome::Gda::DataModel> result;
-         
+
         if(!primary_key_is_empty)
-          result = Query_execute(query);
+          result = Query_execute(query, get_app_window());
 
         if((result && result->get_n_rows()) || primary_key_is_empty) //either a working result or no result needed.
         {
@@ -230,7 +238,7 @@ bool Box_Data_Details::fill_from_database()
           if(pDoc)
           {
             //Get glom-specific field info:
-            //Document_Glom::type_vecFields vecFields = pDoc->get_table_fields(m_strTableName);
+            //Document_Glom::type_vecFields vecFields = pDoc->get_table_fields(m_table_name);
 
             const int row_number = 0; //The only row.
             int cols_count = 0;
@@ -253,15 +261,15 @@ bool Box_Data_Details::fill_from_database()
 
               //Field value:
               Gnome::Gda::Value value;
-               
+
               if(!primary_key_is_empty)
                 value = result->get_value_at(i, row_number);
               else
               {
-                value = GlomConversions::get_empty_value(layout_item->m_field.get_glom_type());
+                value = GlomConversions::get_empty_value(layout_item->get_glom_type());
               }
-               
-              m_FlowTable.set_field_value(*layout_item, value);
+
+              m_FlowTable.set_field_value(layout_item, value);
             }
           }
         }
@@ -292,13 +300,13 @@ void Box_Data_Details::on_button_new()
 {
   if(confirm_discard_unstored_data())
   {
-    if(m_field_primary_key.get_auto_increment()) //If the primary key is an auto-increment:
+    if(m_field_primary_key->get_auto_increment()) //If the primary key is an auto-increment:
     {
       //Just make a new record, and show it:
-      Gnome::Gda::Value primary_key_value = generate_next_auto_increment(m_strTableName, m_field_primary_key.get_name()); //TODO: This should return a Gda::Value
+      Gnome::Gda::Value primary_key_value = generate_next_auto_increment(m_table_name, m_field_primary_key->get_name()); //TODO: This should return a Gda::Value
 
       record_new(false /* use entered field data */, primary_key_value);
-      refresh_data_from_database(primary_key_value);
+      refresh_data_from_database_with_primary_key(primary_key_value);
     }
     else
     {
@@ -360,16 +368,21 @@ void Box_Data_Details::on_button_nav_last()
     signal_nav_last().emit();
 }
 
-Gnome::Gda::Value Box_Data_Details::get_entered_field_data(const LayoutItem_Field& field) const
+Gnome::Gda::Value Box_Data_Details::get_entered_field_data(const sharedptr<const LayoutItem_Field>& field) const
 {
   return m_FlowTable.get_field_value(field);
 }
 
-void Box_Data_Details::set_entered_field_data(const LayoutItem_Field& field, const Gnome::Gda::Value& value)
+void Box_Data_Details::set_entered_field_data(const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value)
 {
   m_FlowTable.set_field_value(field, value);
 }
 
+void Box_Data_Details::set_entered_field_data(const Gtk::TreeModel::iterator& /* row */, const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value)
+{
+  set_entered_field_data(field, value);
+}
+
 Gnome::Gda::Value Box_Data_Details::get_primary_key_value_selected()
 {
   return m_primary_key_value;
@@ -380,25 +393,27 @@ void Box_Data_Details::recalculate_fields_for_related_records(const Glib::ustrin
   m_FieldsCalculationInProgress.clear();
 
   const Gnome::Gda::Value primary_key_value = get_primary_key_value();
-
   for(type_vecFields::iterator iter = m_TableFields.begin(); iter != m_TableFields.end(); ++iter)
   {
-    const Field& field = *iter;
+    const sharedptr<const Field> field = *iter;
 
     //Is this field triggered by this relationship?
-    const Field::type_list_strings triggered_by = field.get_calculation_relationships();
+    const Field::type_list_strings triggered_by = field->get_calculation_relationships();
     Field::type_list_strings::const_iterator iterFind = std::find(triggered_by.begin(), triggered_by.end(), relationship_name);
     if(iterFind != triggered_by.end()) //If it was found
     {
-      const Field& field = *iter;
+      sharedptr<Field> field = *iter;
+      if(field)
+      {
+        FieldInRecord field_in_record(m_table_name, field, m_field_primary_key, primary_key_value);
+        calculate_field(field_in_record); //And any dependencies.
 
-      calculate_field(field, m_field_primary_key, primary_key_value); //And any dependencies.
+        //Calculate anything that depends on this.
+        //sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+        //layout_item->set_full_field_details(field);
 
-      //Calculate anything that depends on this.
-      LayoutItem_Field layout_item;
-      layout_item.m_field = field;
-
-      do_calculations(layout_item, m_field_primary_key, primary_key_value, false /* recurse, reusing m_FieldsCalculationInProgress */);
+        do_calculations(field_in_record, false /* recurse, reusing m_FieldsCalculationInProgress */);
+      }
     }
   }
 
@@ -468,14 +483,15 @@ Box_Data_Details::type_signal_requested_related_details Box_Data_Details::signal
 void Box_Data_Details::on_flowtable_layout_changed()
 {
   //Get new layout:
-  Document_Glom::type_mapLayoutGroupSequence layout_groups;
-  m_FlowTable.get_layout_groups(layout_groups);
-  
+  //Document_Glom::type_mapLayoutGroupSequence layout_groups;
+  //m_FlowTable.get_layout_groups(layout_groups);
+
   //Store it in the document:
   Document_Glom* document = get_document();
-  if(document)
-    document->set_data_layout_groups(m_layout_name, m_strTableName, layout_groups);
-  
+  document->set_modified();
+  //if(document)
+  //  document->set_data_layout_groups(m_layout_name, m_table_name, layout_groups);
+
   //Build the view again from the new layout:
   create_layout();
 
@@ -487,7 +503,7 @@ void Box_Data_Details::on_flowtable_requested_related_details(const Glib::ustrin
 {
   if(GlomConversions::value_is_empty(primary_key_value))
     return; //Ignore empty ID fields.
-    
+
   signal_requested_related_details().emit(table_name, primary_key_value);
 }
 
@@ -496,34 +512,45 @@ void Box_Data_Details::on_flowtable_related_record_changed(const Glib::ustring&
   recalculate_fields_for_related_records(relationship_name);
 }
 
-void Box_Data_Details::on_flowtable_field_open_details_requested(const LayoutItem_Field& layout_field, const Gnome::Gda::Value& field_value)
+void Box_Data_Details::on_flowtable_field_open_details_requested(const sharedptr<const LayoutItem_Field>& layout_field, const Gnome::Gda::Value& field_value)
 {
   if(GlomConversions::value_is_empty(field_value))
     return; //Ignore empty ID fields.
-    
-  Relationship relationship;
-  const bool related_to_one = get_document()->get_field_used_in_relationship_to_one(m_strTableName, layout_field.get_name(), relationship);
-  if(related_to_one)
+
+  sharedptr<Relationship> relationship = get_document()->get_field_used_in_relationship_to_one(m_table_name, layout_field->get_name());
+  if(relationship)
   {
-    signal_requested_related_details().emit(relationship.get_to_table(), field_value);
+    signal_requested_related_details().emit(relationship->get_to_table(), field_value);
   }
 }
 
-void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_field, const Gnome::Gda::Value& field_value)
+void Box_Data_Details::on_flowtable_script_button_clicked(const sharedptr<const LayoutItem_Button>& layout_item)
+{
+  if(layout_item)
+  {
+    const Gnome::Gda::Value primary_key_value = get_primary_key_value();
+    const type_map_fields field_values = get_record_field_values(m_table_name, m_field_primary_key, primary_key_value);
+
+    glom_execute_python_function_implementation(layout_item->get_script(), field_values, //TODO: Maybe use the field's type here.
+    get_document(), get_table_name());
+  }
+}
+
+void Box_Data_Details::on_flowtable_field_edited(const sharedptr<const LayoutItem_Field>& layout_field, const Gnome::Gda::Value& field_value)
 {
   if(m_ignore_signals)
     return;
 
-  const Glib::ustring strFieldName = layout_field.get_name();
+  const Glib::ustring strFieldName = layout_field->get_name();
 
   Gnome::Gda::Value primary_key_value = get_primary_key_value();
   if(!GlomConversions::value_is_empty(primary_key_value)) //If there is not a primary key value:
   {
     Glib::ustring table_name;
-    Field primary_key_field;
+    sharedptr<Field> primary_key_field;
     Gnome::Gda::Value primary_key_value;
 
-    if(!layout_field.get_has_relationship_name())
+    if(!layout_field->get_has_relationship_name())
     {
       table_name = get_table_name();
       primary_key_field = m_field_primary_key;
@@ -533,27 +560,26 @@ void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_
     {
       //If it's a related field then discover the actual table that it's in,
       //plus how to identify the record in that table.
-      const Glib::ustring relationship_name = layout_field.get_relationship_name();
+      const Glib::ustring relationship_name = layout_field->get_relationship_name();
 
       Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
 
-      Relationship relationship;
-      bool test = document->get_relationship(get_table_name(), relationship_name, relationship);
-      if(test)
+      sharedptr<Relationship> relationship = document->get_relationship(get_table_name(), relationship_name);
+      if(relationship)
       {
-        table_name = relationship.get_to_table();
-        const Glib::ustring to_field_name = relationship.get_to_field();
+        table_name = relationship->get_to_table();
+        const Glib::ustring to_field_name = relationship->get_to_field();
         //Get the key field in the other table (the table that we will change)
-        bool test = get_fields_for_table_one_field(table_name, to_field_name, primary_key_field); //TODO_Performance.
-        if(test)
+        primary_key_field = get_fields_for_table_one_field(table_name, to_field_name); //TODO_Performance.
+        if(primary_key_field)
         {
           //Get the value of the corresponding key in the current table (that identifies the record in the table that we will change)
-          LayoutItem_Field layout_item;
-          layout_item.set_name(relationship.get_from_field());
+          sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+          layout_item->set_full_field_details( document->get_field(relationship->get_from_table(), relationship->get_from_field()) );
 
           primary_key_value = get_entered_field_data(layout_item);
 
-          bool test = add_related_record_for_field(layout_field, relationship, primary_key_field, primary_key_value);
+          const bool test = add_related_record_for_field(layout_field, relationship, primary_key_field, primary_key_value);
           if(!test)
             return;
 
@@ -569,18 +595,19 @@ void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_
       }
     }
 
-    //const Field& field = layout_field.m_field;
+    //const sharedptr<const Field>& field = layout_field->m_field;
 
     //Set the value in all instances of this field in the layout (The field might be on the layout more than once):
-    if(layout_field.m_field.get_glom_type() != Field::TYPE_IMAGE) //TODO For now, don't do this for images, because the ImageGlom widget expects a broken GdaValue, because gda_value_get_binary() needs to be fixed.
+    //if(layout_field->get_glom_type() != Field::TYPE_IMAGE) //TODO For now, don't do this for images, because the ImageGlom widget expects a broken GdaValue, because gda_value_get_binary() needs to be fixed.
       m_FlowTable.set_field_value(layout_field, field_value);
 
     //Update the field in the record (the record with this primary key):
     try
     {
-      bool bTest = set_field_value_in_database(layout_field, field_value, primary_key_field, primary_key_value);
+      FieldInRecord field_in_record(layout_field, m_table_name /* parent table */, primary_key_field, primary_key_value, *(get_document()));
+      const bool bTest = set_field_value_in_database(field_in_record, field_value, false /* don't use current calculations */, get_app_window());
 
-      //Glib::ustring strQuery = "UPDATE " + table_name;
+      //Glib::ustring strQuery = "UPDATE \"" + table_name + "\"";
       //strQuery += " SET " +  /* table_name + "." + postgres does not seem to like the table name here */ strFieldName + " = " + field.sql(field_value);
       //strQuery += " WHERE " + table_name + "." + primary_key_field.get_name() + " = " + primary_key_field.sql(primary_key_value);
       //bool bTest = Query_execute(strQuery);
@@ -597,12 +624,12 @@ void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_
         //If this is a foreign key then refresh the related records:
         /*
         bool bIsForeignKey = false;
-        Document_Glom::type_vecRelationships vecRelationships = get_document()->get_relationships(m_strTableName);
+        Document_Glom::type_vecRelationships vecRelationships = get_document()->get_relationships(m_table_name);
         for(Document_Glom::type_vecRelationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); iter++)
         {
           const Relationship& relationship = *iter;
 
-          if(relationship.get_from_field() == strFieldName)
+          if(relationship->get_from_field() == strFieldName)
           {
             bIsForeignKey = true;
             break;
@@ -623,10 +650,10 @@ void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_
   else
   {
     //There is no current primary key value:
-    
-    if(m_field_primary_key.get_auto_increment()) //If the primary key is an auto-increment:
+
+    if(m_field_primary_key->get_auto_increment()) //If the primary key is an auto-increment:
     {
-      if(strFieldName == m_field_primary_key.get_name()) //If edited field is the primary key.
+      if(strFieldName == m_field_primary_key->get_name()) //If edited field is the primary key.
       {
         //Warn user that they can't choose their own primary key:
         Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Primary key auto increments")), true);
@@ -637,17 +664,17 @@ void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_
       else
       {
         //Make a new record, and show it:
-        Gnome::Gda::Value primary_key_value = generate_next_auto_increment(m_strTableName, m_field_primary_key.get_name());
+        Gnome::Gda::Value primary_key_value = generate_next_auto_increment(m_table_name, m_field_primary_key->get_name());
 
         record_new(true /* use entered field data */, primary_key_value);
-        refresh_data_from_database(primary_key_value);
+        refresh_data_from_database_with_primary_key(primary_key_value);
       }
     }
     else
     {
       //It is not auto-generated:
 
-      if(strFieldName == m_field_primary_key.get_name()) //if it is the primary key that' being edited.
+      if(strFieldName == m_field_primary_key->get_name()) //if it is the primary key that' being edited.
       {
         //Create new record with this primary key,
         //and all the other field values too.
@@ -666,136 +693,57 @@ void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_
   } //if(get_primary_key_value().size())
 }
 
-void Box_Data_Details::refresh_related_fields(const Gtk::TreeModel::iterator& /* row */, const LayoutItem_Field& field_changed, const Gnome::Gda::Value& /* field_value */, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
-{
-  if(field_changed.get_has_relationship_name())
-    return; //TODO: Handle these too.
-
-  //Get values for lookup fields, if this field triggers those relationships:
-  //TODO_performance: There is a LOT of iterating and copying here.
-  const Glib::ustring strFieldName = field_changed.get_name();
-  type_vecLayoutFields fieldsToGet = get_related_fields(strFieldName);
-
-  if(!fieldsToGet.empty())
-  {
-    const Glib::ustring query = build_sql_select(m_strTableName, fieldsToGet, primary_key, primary_key_value);
-
-    //g_warning("Box_Data_Details::refresh_related_fields(): get query = %s", query.c_str());
-    
-    Glib::RefPtr<Gnome::Gda::DataModel> result = Query_execute(query);
-    if(!result)
-      handle_error();
-    else
-    {
-      //Field contents:
-      if(result->get_n_rows())
-      {
-        type_vecLayoutFields::const_iterator iterFields = fieldsToGet.begin();
-
-        guint cols_count = result->get_n_columns();
-        for(guint uiCol = 0; uiCol < cols_count; uiCol++)
-        {
-          const Gnome::Gda::Value value = result->get_value_at(uiCol, 0 /* row */);
-          sharedptr<LayoutItem_Field> layout_item = *iterFields;
-
-          //g_warning("list fill: field_name=%s", iterFields->get_name().c_str());
-          //g_warning("  value_as_string=%s", value.to_string().c_str());
-
-          m_FlowTable.set_field_value(*layout_item, value);
-
-          ++iterFields;
-        }
-      }
-    }
-  }
-}
-
-void Box_Data_Details::do_lookups(const Gtk::TreeModel::iterator& /* row */, const LayoutItem_Field& field_changed, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
-{
-   if(field_changed.get_has_relationship_name())
-    return; //TODO: Handle these too.
-
-   //Get values for lookup fields, if this field triggers those relationships:
-   //TODO_performance: There is a LOT of iterating and copying here.
-   const Glib::ustring strFieldName = field_changed.get_name();
-   const type_list_lookups lookups = get_lookup_fields(strFieldName);
-   for(type_list_lookups::const_iterator iter = lookups.begin(); iter != lookups.end(); ++iter)
-   {
-     const LayoutItem_Field& layout_item = iter->first;
-
-     const Relationship relationship = iter->second;
-     const Field& field_lookup = layout_item.m_field;
-     const Glib::ustring field_lookup_name = field_lookup.get_name();
-
-     Field field_source;
-     bool test = get_fields_for_table_one_field(relationship.get_to_table(), field_lookup.get_lookup_field(), field_source);
-     if(test)
-     {
-       Gnome::Gda::Value value = get_lookup_value(iter->second /* relationship */,  field_source /* the field to look in to get the value */, field_value /* Value of to and from fields */);
-
-       //Add it to the view:
-       //TODO? layout_item.set_relationship_name();
-       set_entered_field_data(layout_item, value);
-
-       //Add it to the database (even if it is not shown in the view)
-       set_field_value_in_database(layout_item, value, primary_key, primary_key_value);
-     }
-   }
-}
-
 void Box_Data_Details::on_userlevel_changed(AppState::userlevels user_level)
 {
   m_FlowTable.set_design_mode( user_level == AppState::USERLEVEL_DEVELOPER );
 }
 
-bool Box_Data_Details::get_field_primary_key(Field& field) const
+sharedptr<Field> Box_Data_Details::get_field_primary_key() const
 {
-  field = m_field_primary_key;
-  return true;
+  return m_field_primary_key;
 }
 
-void Box_Data_Details::print_layout_group(xmlpp::Element* node_parent, const LayoutGroup& group)
+void Box_Data_Details::print_layout_group(xmlpp::Element* node_parent, const sharedptr<const LayoutGroup>& group)
 {
   xmlpp::Element* nodeChildGroup = node_parent->add_child("group");
-  nodeChildGroup->set_attribute("title", group.m_title);
+  nodeChildGroup->set_attribute("title", group->get_title());
 
-  LayoutGroup::type_map_const_items items = group.get_items();
+  LayoutGroup::type_map_const_items items = group->get_items();
   for(LayoutGroup::type_map_const_items::const_iterator iter = items.begin(); iter != items.end(); ++iter)
   {
-    const LayoutItem* layout_item = iter->second;
+    sharedptr<const LayoutItem> layout_item = iter->second;
 
-    const LayoutGroup* pLayoutGroup = dynamic_cast<const LayoutGroup*>(layout_item);
+    sharedptr<const LayoutGroup> pLayoutGroup = sharedptr<const LayoutGroup>::cast_dynamic(layout_item);
     if(pLayoutGroup)
-      print_layout_group(nodeChildGroup, *pLayoutGroup); //recurse
+      print_layout_group(nodeChildGroup, pLayoutGroup); //recurse
     else
     {
-      const LayoutItem_Field* pLayoutField = dynamic_cast<const LayoutItem_Field*>(layout_item);
+      sharedptr<const LayoutItem_Field> pLayoutField = sharedptr<const LayoutItem_Field>::cast_dynamic(layout_item);
       if(pLayoutField)
       {
         xmlpp::Element* nodeField = nodeChildGroup->add_child("field");
 
         nodeField->set_attribute("title", pLayoutField->get_title_or_name());
 
-        Gnome::Gda::Value value = m_FlowTable.get_field_value(*pLayoutField);
-        Glib::ustring text_representation = GlomConversions::get_text_for_gda_value(pLayoutField->m_field.get_glom_type(), value,
+        Gnome::Gda::Value value = m_FlowTable.get_field_value(pLayoutField);
+        const Glib::ustring text_representation = GlomConversions::get_text_for_gda_value(pLayoutField->get_glom_type(), value,
           pLayoutField->get_formatting_used().m_numeric_format); //In the current locale.
 
         nodeField->set_attribute("value", text_representation);
       }
       else
       {
-        const LayoutItem_Portal* pLayoutPortal = dynamic_cast<const LayoutItem_Portal*>(layout_item);
+        sharedptr<const LayoutItem_Portal> pLayoutPortal = sharedptr<const LayoutItem_Portal>::cast_dynamic(layout_item);
         if(pLayoutPortal)
         {
           xmlpp::Element* nodePortal = nodeChildGroup->add_child("related_records");
 
           //Box_Data_List_Related* pPortalWidget = m_FlowTable.get_portals();
 
-          Relationship relationship;
-          bool relationship_found = get_document()->get_relationship(m_strTableName, pLayoutPortal->get_relationship(), relationship);
-          if(relationship_found)
+          sharedptr<Relationship> relationship = get_document()->get_relationship(m_table_name, pLayoutPortal->get_relationship_name());
+          if(relationship)
           {
-            nodePortal->set_attribute("title", relationship.get_title_or_name());
+            nodePortal->set_attribute("title", relationship->get_title_or_name());
 
             //TODO:
 
@@ -810,7 +758,7 @@ void Box_Data_Details::print_layout_group(xmlpp::Element* node_parent, const Lay
 
 void Box_Data_Details::print_layout()
 {
-  const Privileges table_privs = get_current_privs(m_strTableName);
+  const Privileges table_privs = get_current_privs(m_table_name);
 
   //Don't try to print tables that the user can't view.
   if(!table_privs.m_view)
@@ -830,9 +778,9 @@ void Box_Data_Details::print_layout()
       nodeRoot = pDocument->create_root_node("details_print");
     }
 
-    Glib::ustring table_title = get_document()->get_table_title(m_strTableName);
+    Glib::ustring table_title = get_document()->get_table_title(m_table_name);
     if(table_title.empty())
-      table_title = m_strTableName;
+      table_title = m_table_name;
 
     nodeRoot->set_attribute("table", table_title);
 
@@ -843,10 +791,10 @@ void Box_Data_Details::print_layout()
     Document_Glom::type_mapLayoutGroupSequence layout_groups = get_data_layout_groups(m_layout_name);
     for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
     {
-      const LayoutGroup& layout_group = iter->second;
+      sharedptr<const LayoutGroup> layout_group = iter->second;
       print_layout_group(nodeParent, layout_group);
     }
 
-    GlomUtils::transform_and_open(*pDocument, "print_details_to_html.xsl");
+    GlomXslUtils::transform_and_open(*pDocument, "print_details_to_html.xsl", get_app_window());
   }
 }
