@@ -19,14 +19,16 @@
  */
 
 #include "box_data_list.h"
-#include "../data_structure/glomconversions.h"
+#include <glom/libglom/data_structure/glomconversions.h>
 #include "dialog_layout_list.h"
 #include <bakery/App/App_Gtk.h> //For util_bold_message().
+//#include <../utility_widgets/db_adddel/glom_db_treemodel.h> //For DbTreeModel.
 #include <sstream> //For stringstream
 #include <glibmm/i18n.h>
 
 Box_Data_List::Box_Data_List()
-: m_has_one_or_more_records(false)
+: m_has_one_or_more_records(false),
+  m_read_only(false)
 {
   m_layout_name = "list";
 
@@ -74,23 +76,42 @@ Box_Data_List::~Box_Data_List()
 
 void Box_Data_List::enable_buttons()
 {
-  const Privileges table_privs = get_current_privs(m_strTableName);
-  
+  const Privileges table_privs = get_current_privs(m_table_name);
+
     //Enable/Disable record creation and deletion:
-  m_AddDel.set_allow_add(table_privs.m_create);
-  m_AddDel.set_allow_delete(table_privs.m_delete);
+  bool allow_create = !m_read_only;
+  bool allow_delete = !m_read_only;
+  if(!m_read_only)
+  {
+    allow_create = table_privs.m_create;
+    allow_delete = table_privs.m_delete;
+  }
+
+  m_AddDel.set_allow_add(allow_create);
+  m_AddDel.set_allow_delete(allow_delete);
+
   m_AddDel.set_allow_view_details(table_privs.m_view);
 }
 
+void Box_Data_List::refresh_data_from_database_blank()
+{
+  FoundSet found_set = m_found_set;
+  found_set.m_where_clause = Glib::ustring();
+  m_AddDel.set_found_set(found_set);
+
+  m_AddDel.refresh_from_database_blank();
+  m_found_set = found_set;
+}
+
 bool Box_Data_List::fill_from_database()
 {
   bool result = false;
 
-  Bakery::BusyCursor(*get_app_window());
+  Bakery::BusyCursor busy_cursor(get_app_window());
 
   try
   {
-    sharedptr<SharedConnection> sharedconnection = connect_to_server();
+    sharedptr<SharedConnection> sharedconnection = connect_to_server(get_app_window());
 
     Box_Data::fill_from_database();
 
@@ -104,11 +125,11 @@ bool Box_Data_List::fill_from_database()
       //Glib::RefPtr<Gnome::Gda::Connection> connection = sharedconnection->get_gda_connection();
 
     //Do not try to show the data if the user may not view it:
-    const Privileges table_privs = get_current_privs(m_strTableName);
+    const Privileges table_privs = get_current_privs(m_table_name);
 
     enable_buttons();
 
-    m_AddDel.set_where_clause(m_strWhereClause);
+    m_AddDel.set_found_set(m_found_set);
 
     result = m_AddDel.refresh_from_database();
 
@@ -125,7 +146,7 @@ bool Box_Data_List::fill_from_database()
 
     fill_end();
   }
-  catch(std::exception& ex)
+  catch(const std::exception& ex)
   {
     handle_error(ex);
     result = false;
@@ -136,10 +157,9 @@ bool Box_Data_List::fill_from_database()
 
 void Box_Data_List::on_adddel_user_requested_add()
 {
-g_warning("Box_Data_List::on_adddel_user_requested_add() start: m_FieldsShowns.size()=%d", m_FieldsShown.size());
-  if(m_FieldsShown.empty())
+ if(m_FieldsShown.empty())
     return; //Don't try to add a record to a list with no fields.
-    
+
   Gtk::TreeModel::iterator iter = m_AddDel.get_item_placeholder();
   if(iter)
   {
@@ -151,9 +171,8 @@ g_warning("Box_Data_List::on_adddel_user_requested_add() start: m_FieldsShowns.s
     {
       index_field_to_edit = index_primary_key;
 
-      Field fieldPrimaryKey;
-      bool test = get_field_primary_key(fieldPrimaryKey);
-      if(test && fieldPrimaryKey.get_auto_increment())
+      sharedptr<Field> fieldPrimaryKey = get_field_primary_key();
+      if(fieldPrimaryKey && fieldPrimaryKey->get_auto_increment())
       {
         //Start editing in the first cell that is not the primary key:
         if(index_primary_key == 0)
@@ -170,18 +189,20 @@ g_warning("Box_Data_List::on_adddel_user_requested_add() start: m_FieldsShowns.s
       guint treemodel_column = 0;
       bool test = get_field_column_index(m_FieldsShown[index_field_to_edit]->get_name(), treemodel_column);
       if(test)
+      {
+        //std::cout << "on_adddel_user_requested_add(): editing column=" << treemodel_column << std::endl;
         m_AddDel.select_item(iter, treemodel_column, true /* start_editing */);
+      }
     }
     else
     {
+      std::cout << "on_adddel_user_requested_add(): no editable rows." << std::endl;
       //The only keys are non-editable, so just add a row:
       on_adddel_user_added(iter, 0);
       m_AddDel.select_item(iter); //without start_editing.
       //g_warning("Box_Data_List::on_adddel_user_requested_add(): index_field_to_edit does not exist: %d", index_field_to_edit);
     }
   }
-  
-  g_warning("Box_Data_List::on_adddel_user_requested_add() end");
 }
 
 void Box_Data_List::on_adddel_user_requested_edit(const Gtk::TreeModel::iterator& row)
@@ -200,15 +221,7 @@ void Box_Data_List::on_adddel_user_requested_delete(const Gtk::TreeModel::iterat
 {
   if(rowStart)
   {
-    //Ask the user for confirmation:
-    Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Delete record")), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE);
-    dialog.set_secondary_text(_("Are you sure that you would like to delete this record? The data in this record will then be permanently lost."));
-    dialog.set_transient_for(*get_app_window());
-    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
-    dialog.add_button(Gtk::Stock::DELETE, Gtk::RESPONSE_OK);
-
-    int response = dialog.run();
-    if(response == Gtk::RESPONSE_OK)
+    if(confirm_delete_record())
     {
       const Gnome::Gda::Value primary_key_value = get_primary_key_value(rowStart);
       record_delete(primary_key_value);
@@ -223,29 +236,29 @@ void Box_Data_List::on_adddel_user_requested_delete(const Gtk::TreeModel::iterat
 
 void Box_Data_List::on_adddel_user_added(const Gtk::TreeModel::iterator& row, guint col_with_first_value)
 {
+  //std::cout << "Box_Data_List::on_adddel_user_added" << std::endl;
+
   Gnome::Gda::Value primary_key_value;
 
-  Field field_primary_key = m_AddDel.get_key_field();
+  sharedptr<Field> field_primary_key = m_AddDel.get_key_field();
 
   //Get the new primary key value, if one is available now:
-  if(field_primary_key.get_auto_increment())
+  if(field_primary_key->get_auto_increment())
   {
     //Auto-increment is awkward (we can't get the last-generated ID) with postgres, so we auto-generate it ourselves;
-    const Glib::ustring& strPrimaryKeyName = field_primary_key.get_name();
-    primary_key_value = generate_next_auto_increment(m_strTableName, strPrimaryKeyName);  //TODO: return a Gnome::Gda::Value of an appropriate type.
+    const Glib::ustring& strPrimaryKeyName = field_primary_key->get_name();
+    primary_key_value = generate_next_auto_increment(m_table_name, strPrimaryKeyName);  //TODO: return a Gnome::Gda::Value of an appropriate type.
   }
   else
   {
     //This only works when the primary key is already stored: primary_key_value = get_primary_key_value(row);
-    LayoutItem_Field layout_item;
-    layout_item.m_field = field_primary_key;
-    primary_key_value = get_entered_field_data(layout_item);
+    primary_key_value = get_entered_field_data_field_only(field_primary_key);
   }
 
   //If no primary key value is available yet, then don't add the record yet:
   if(!GlomConversions::value_is_empty(primary_key_value))
   {
-    sharedptr<SharedConnection> sharedconnection = connect_to_server(); //Keep it alive while we need the data_model.
+    sharedptr<SharedConnection> sharedconnection = connect_to_server(get_app_window()); //Keep it alive while we need the data_model.
     if(sharedconnection)
     {
       Glib::RefPtr<Gnome::Gda::DataModel> data_model = record_new(true /* use entered field data*/, primary_key_value);
@@ -257,10 +270,10 @@ void Box_Data_List::on_adddel_user_added(const Gtk::TreeModel::iterator& row, gu
         //Show the primary key in the row, if the primary key is visible:
 
         //If it's an auto-increment, then get the value and show it:
-        if(field_primary_key.get_auto_increment())
+        if(field_primary_key->get_auto_increment())
         {
-          LayoutItem_Field layout_item;
-          layout_item.m_field = field_primary_key;
+          sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+          layout_item->set_full_field_details(field_primary_key);
           m_AddDel.set_value(row, layout_item, primary_key_value);
         }
 
@@ -286,25 +299,25 @@ void Box_Data_List::on_adddel_user_reordered_columns()
   Document_Glom* pDoc = dynamic_cast<Document_Glom*>(get_document());
   if(pDoc)
   {
-    LayoutGroup group;
-    group.set_name("toplevel");
+    sharedptr<LayoutGroup> group = sharedptr<LayoutGroup>::create();
+    group->set_name("toplevel");
 
     AddDel::type_vecStrings vec_field_names = m_AddDel.get_columns_order();
 
     guint index = 0;
     for(AddDel::type_vecStrings::iterator iter = vec_field_names.begin(); iter != vec_field_names.end(); ++iter)
     {
-      LayoutItem_Field layout_item;
-      layout_item.set_name(*iter);
-      layout_item.m_sequence = index;
-      group.add_item(layout_item, index); 
+      sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+      layout_item->set_name(*iter);
+      layout_item->m_sequence = index;
+      group->add_item(layout_item, index); 
       ++index;
     }
 
     Document_Glom::type_mapLayoutGroupSequence mapGroups;
     mapGroups[1] = group;
 
-    pDoc->set_data_layout_groups("list", m_strTableName, mapGroups);  
+    pDoc->set_data_layout_groups("list", m_table_name, mapGroups);  
   }
 }
 
@@ -317,15 +330,15 @@ void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row,
     //Just update the record:
     try
     {
-      LayoutItem_Field layout_field = m_AddDel.get_column_field(col);
+      sharedptr<const LayoutItem_Field> layout_field = m_AddDel.get_column_field(col);
 
-      Glib::ustring table_name = m_strTableName;
-      Field primary_key_field;
+      Glib::ustring table_name = m_table_name;
+      sharedptr<Field> primary_key_field;
       Gnome::Gda::Value primary_key_value;
 
-      if(!layout_field.get_has_relationship_name())
+      if(!layout_field->get_has_relationship_name())
       {
-        table_name = m_strTableName;
+        table_name = m_table_name;
         primary_key_field = m_AddDel.get_key_field();
         primary_key_value = parent_primary_key_value;
       }
@@ -333,27 +346,26 @@ void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row,
       {
         //If it's a related field then discover the actual table that it's in,
         //plus how to identify the record in that table.
-        const Glib::ustring relationship_name = layout_field.get_relationship_name();
+        const Glib::ustring relationship_name = layout_field->get_relationship_name();
 
         Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
 
-        Relationship relationship;
-        bool test = document->get_relationship(m_strTableName, relationship_name, relationship);
-        if(test)
+        sharedptr<Relationship> relationship = document->get_relationship(m_table_name, relationship_name);
+        if(relationship)
         {
-          table_name = relationship.get_to_table();
-          const Glib::ustring to_field_name = relationship.get_to_field();
+          table_name = relationship->get_to_table();
+          const Glib::ustring to_field_name = relationship->get_to_field();
           //Get the key field in the other table (the table that we will change)
-          bool test = get_fields_for_table_one_field(table_name, to_field_name, primary_key_field); //TODO_Performance.
-          if(test)
+          primary_key_field = get_fields_for_table_one_field(table_name, to_field_name); //TODO_Performance.
+          if(primary_key_field)
           {
             //Get the value of the corresponding key in the current table (that identifies the record in the table that we will change)
-            LayoutItem_Field layout_item;
-            layout_item.set_name(relationship.get_from_field());
+            sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
+            layout_item->set_name(relationship->get_from_field());
 
             primary_key_value = get_entered_field_data(layout_item);
 
-            bool test = add_related_record_for_field(layout_field, relationship, primary_key_field, primary_key_value);
+            const bool test = add_related_record_for_field(layout_field, relationship, primary_key_field, primary_key_value);
             if(!test)
               return;
 
@@ -371,12 +383,14 @@ void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row,
 
       //Update the field in the record (the record with this primary key):
       const Gnome::Gda::Value field_value = m_AddDel.get_value(row, layout_field);
-      //const Field& field = layout_field.m_field;
-      //const Glib::ustring strFieldName = layout_field.get_name();
+      //std::cout << "Box_Data_List::on_adddel_user_changed(): field_value = " << field_value.to_string() << std::endl;
+      //const sharedptr<const Field>& field = layout_field->m_field;
+      //const Glib::ustring strFieldName = layout_field->get_name();
 
-      bool bTest = set_field_value_in_database(row, layout_field, field_value, primary_key_field, primary_key_value);
+      FieldInRecord field_in_record(layout_field, m_table_name /* parent */, primary_key_field, primary_key_value, *(get_document()));
+      const bool bTest = set_field_value_in_database(field_in_record, row, field_value, false /* don't use current calculations */, get_app_window());
 
-      //Glib::ustring strQuery = "UPDATE " + table_name;
+      //Glib::ustring strQuery = "UPDATE \"" + table_name + "\"";
       //strQuery += " SET " +  /* table_name + "." + postgres does not seem to like the table name here */ strFieldName + " = " + field.sql(field_value);
       //strQuery += " WHERE " + table_name + "." + primary_key_field.get_name() + " = " + primary_key_field.sql(primary_key_value);
       //bool bTest = Query_execute(strQuery);
@@ -392,7 +406,7 @@ void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row,
         do_lookups(row, layout_field, field_value, primary_key_field, primary_key_value);
 
         //Recalculate calculated fields, if this field is used by them:
-        do_calculations(layout_field, primary_key_field, primary_key_value);
+        do_calculations(m_table_name, layout_field, primary_key_field, primary_key_value);
 
         //Update related fields, if this field is used in the relationship:
         refresh_related_fields(row, layout_field, field_value, primary_key_field, primary_key_value);
@@ -427,92 +441,7 @@ void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row,
 
 }
 
-void Box_Data_List::refresh_related_fields(const Gtk::TreeModel::iterator& row, const LayoutItem_Field& field_changed, const Gnome::Gda::Value& /* field_value */, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
-{
-  if(field_changed.get_has_relationship_name())
-    return; //TODO: Handle these too.
 
-  //Get values for lookup fields, if this field triggers those relationships:
-  //TODO_performance: There is a LOT of iterating and copying here.
-  const Glib::ustring strFieldName = field_changed.get_name();
-  type_vecLayoutFields fieldsToGet = get_related_fields(strFieldName);
-
-  if(!fieldsToGet.empty())
-  {
-    const Glib::ustring query = build_sql_select(m_strTableName, fieldsToGet, primary_key, primary_key_value);
-
-    Glib::RefPtr<Gnome::Gda::DataModel> result = Query_execute(query);
-    if(!result)
-    {
-      g_warning("Box_Data_List::refresh_related_fields(): no result.");
-      handle_error();
-    }
-    else
-    {
-      //Field contents:
-      if(result->get_n_rows())
-      {
-        type_vecLayoutFields::const_iterator iterFields = fieldsToGet.begin();
-
-        guint cols_count = result->get_n_columns();
-        for(guint uiCol = 0; uiCol < cols_count; uiCol++)
-        {
-          const Gnome::Gda::Value value = result->get_value_at(uiCol, 0 /* row */);
-          sharedptr<LayoutItem_Field> layout_item = *iterFields;
-
-          //g_warning("list fill: field_name=%s", iterFields->get_name().c_str());
-          //g_warning("  value_as_string=%s", value.to_string().c_str());
-
-          m_AddDel.set_value(row, *layout_item, value);
-            //g_warning("addedel size=%d", m_AddDel.get_count());
-
-          ++iterFields;
-        }
-      }
-      else
-       g_warning("Box_Data_List::refresh_related_fields(): no records found.");
-    }
-  }
-}
-
-void Box_Data_List::do_lookups(const Gtk::TreeModel::iterator& row, const LayoutItem_Field& field_changed, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value)
-{
-   if(field_changed.get_has_relationship_name())
-    return; //TODO: Handle these too.
-
-
-   //Get values for lookup fields, if this field triggers those relationships:
-   //TODO_performance: There is a LOT of iterating and copying here.
-   const Glib::ustring strFieldName = field_changed.get_name();
-   type_list_lookups lookups = get_lookup_fields(strFieldName);
-   for(type_list_lookups::const_iterator iter = lookups.begin(); iter != lookups.end(); ++iter)
-   {
-     const LayoutItem_Field& layout_Item = iter->first;
-
-     const Relationship relationship = iter->second;
-     const Field& field_lookup = layout_Item.m_field;
-
-     Field field_source;
-     bool test = get_fields_for_table_one_field(relationship.get_to_table(), field_lookup.get_lookup_field(), field_source);
-     if(test)
-     {
-       Gnome::Gda::Value value = get_lookup_value(iter->second /* relationship */,  field_source /* the field to look in to get the value */, field_value /* Value of to and from fields */);
-
-       //Add it to the view:
-       m_AddDel.set_value(row, layout_Item, value);
-
-       //Add it to the database (even if it is not shown in the view)
-       set_field_value_in_database(row, layout_Item, value, primary_key, primary_key_value); //Also does dependent lookups/recalcs.
-       //Glib::ustring strQuery = "UPDATE " + m_strTableName;
-       //strQuery += " SET " + field_lookup.get_name() + " = " + field_lookup.sql(value);
-       //strQuery += " WHERE " + primary_key.get_name() + " = " + primary_key.sql(primary_key_value);
-       //Query_execute(strQuery);  //TODO: Handle errors
-
-       //TODO: Handle lookups triggered by these fields (recursively)? TODO: Check for infinitely looping lookups.
-     }
-   }
-
-}
 
 void Box_Data_List::on_details_nav_first()
 {
@@ -561,9 +490,11 @@ void Box_Data_List::on_details_nav_last()
     m_AddDel.select_item(iter);
     signal_user_requested_details().emit(m_AddDel.get_value_key_selected());
   }
+  else
+    signal_user_requested_details().emit(Gnome::Gda::Value()); //Show a blank record if there are no records.
 }
 
-void Box_Data_List::on_Details_record_deleted(Gnome::Gda::Value primary_key_value)
+void Box_Data_List::on_Details_record_deleted(const Gnome::Gda::Value& primary_key_value)
 {
   //Find out which row is affected:
   Gtk::TreeModel::iterator iter = m_AddDel.get_row(primary_key_value);
@@ -603,22 +534,55 @@ Gnome::Gda::Value Box_Data_List::get_primary_key_value(const Gtk::TreeModel::ite
 
 Gnome::Gda::Value Box_Data_List::get_primary_key_value_selected()
 {
-  return Gnome::Gda::Value(m_AddDel.get_value_key_selected());
+  return m_AddDel.get_value_key_selected();
 }
 
-Gnome::Gda::Value Box_Data_List::get_entered_field_data(const LayoutItem_Field& field) const
+Gnome::Gda::Value Box_Data_List::get_primary_key_value_first()
+{
+  //std::cout << "Box_Data_List(): get_primary_key_value_first() records_count = " << m_AddDel.get_count() << std::endl;
+
+  Glib::RefPtr<Gtk::TreeModel> model = m_AddDel.get_model();
+  if(model)
+  {
+    Gtk::TreeModel::iterator iter = model->children().begin();
+    while(iter != model->children().end())
+    {
+      Gnome::Gda::Value value = get_primary_key_value(iter);
+      if(GlomConversions::value_is_empty(value))
+      {
+       //std::cout << "Box_Data_List(): get_primary_key_value_first() iter val is NULL" << std::endl;
+        ++iter;
+      }
+      else
+      {
+         //std::cout << "Box_Data_List(): get_primary_key_value_first() returning: " << value.to_string() << std::endl;
+        return value;
+      }
+    }
+  }
+
+ // std::cout << "Box_Data_List(): get_primary_key_value_first() return NULL" << std::endl;
+  return Gnome::Gda::Value();
+}
+
+Gnome::Gda::Value Box_Data_List::get_entered_field_data(const sharedptr<const LayoutItem_Field>& field) const
 {
   return m_AddDel.get_value_selected(field);
 }
 
-void Box_Data_List::set_entered_field_data(const LayoutItem_Field& field, const Gnome::Gda::Value& value)
+void Box_Data_List::set_entered_field_data(const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value)
 {
   return m_AddDel.set_value_selected(field, value);
 }
 
-guint Box_Data_List::get_records_count() const
+void Box_Data_List::set_entered_field_data(const Gtk::TreeModel::iterator& row, const sharedptr<const LayoutItem_Field>& field, const Gnome::Gda::Value& value)
 {
-  return m_AddDel.get_count();
+  return m_AddDel.set_value(row, field, value);
+}
+
+bool Box_Data_List::get_showing_multiple_records() const
+{
+  return m_AddDel.get_count() > 1;
 }
 
 void Box_Data_List::create_layout()
@@ -632,11 +596,10 @@ void Box_Data_List::create_layout()
     m_AddDel.remove_all_columns();
     //m_AddDel.set_columns_count(m_Fields.size());
 
-    m_AddDel.set_table_name(m_strTableName);
+    m_AddDel.set_table_name(m_table_name);
 
-    Field field_primary_key;
-    bool test = get_field_primary_key_for_table(m_strTableName, field_primary_key);
-    if(!test)
+    sharedptr<Field> field_primary_key = get_field_primary_key_for_table(m_table_name);
+    if(!field_primary_key)
     {
       //g_warning("Box_Data_List::create_layout(): primary key not found.");
     }
@@ -648,19 +611,24 @@ void Box_Data_List::create_layout()
 
       //Add extra possibly-non-visible columns that we need:
       //TODO: Only add it if it is not already there.
-      sharedptr<LayoutItem_Field> layout_item(new LayoutItem_Field);
+      sharedptr<LayoutItem_Field> layout_item = sharedptr<LayoutItem_Field>::create();
       layout_item->set_hidden();
-      layout_item->m_field = m_AddDel.get_key_field();
+      layout_item->set_full_field_details(m_AddDel.get_key_field());
       m_FieldsShown.push_back(layout_item);
 
 
       //Add a column for each table field:
-      for(type_vecLayoutFields::const_iterator iter = m_FieldsShown.begin(); iter != m_FieldsShown.end(); ++iter)
+      for(type_vecLayoutFields::iterator iter = m_FieldsShown.begin(); iter != m_FieldsShown.end(); ++iter)
       {
-        m_AddDel.add_column(*(*iter));
+        sharedptr<LayoutItem_Field> field = *iter;
+        if(m_read_only)
+          field->set_editable(false);
+
+        //std::cout << "Adding field: name=" << field->get_name() << ", titleorname=" << field->get_title_or_name() << std::endl;
+        m_AddDel.add_column(field);
       }
 
-      m_AddDel.set_where_clause(m_strWhereClause);
+      m_AddDel.set_found_set(m_found_set);
 
       m_AddDel.set_columns_ready();
      }
@@ -704,20 +672,19 @@ bool Box_Data_List::get_field_column_index(const Glib::ustring& field_name, guin
   return false; //failure.
 }
 
-bool Box_Data_List::get_field_primary_key(Field& field) const
+sharedptr<Field> Box_Data_List::get_field_primary_key() const
 {
-  field = m_AddDel.get_key_field();
-  return true;
+  return m_AddDel.get_key_field();
 }
 
 bool Box_Data_List::get_field_primary_key_index(guint& field_column) const
 {
-  return Box_Data::get_field_primary_key_index(m_FieldsShown, field_column);
+  return Box_Data::get_field_primary_key_index_for_fields(m_FieldsShown, field_column);
 }
 
 void Box_Data_List::print_layout()
 {
-  const Privileges table_privs = get_current_privs(m_strTableName);
+  const Privileges table_privs = get_current_privs(m_table_name);
 
   //Don't try to print tables that the user can't view.
   if(!table_privs.m_view)
@@ -727,22 +694,54 @@ void Box_Data_List::print_layout()
   else
   {
     //Create a simple report on the fly:
-    Report report_temp;
-    report_temp.m_name = "list";
-    report_temp.m_title = _("List");
+    sharedptr<Report> report_temp(new Report());
+    report_temp->set_name("list");
+    report_temp->set_title(_("List"));
 
     //Add all the fields from the layout:
     for(type_vecLayoutFields::const_iterator iter = m_FieldsShown.begin(); iter != m_FieldsShown.end(); ++iter)
     {
-      report_temp.m_layout_group.add_item(*(*iter));
+      report_temp->m_layout_group->add_item(*iter);
     }
 
-    report_build(m_strTableName, report_temp, m_strWhereClause);
+    report_build(m_found_set, report_temp, get_app_window());
   }
 }
 
-void Box_Data_List::print_layout_group(xmlpp::Element* /* node_parent */, const LayoutGroup& /* group */)
+void Box_Data_List::print_layout_group(xmlpp::Element* /* node_parent */, const sharedptr<const LayoutGroup>& /* group */)
 {
 }
 
+void Box_Data_List::set_read_only(bool read_only)
+{
+  //This is useful when showing find results for the user to select one, without changing them.
+  m_read_only = read_only;
+  m_AddDel.set_allow_add(!read_only);
+  m_AddDel.set_allow_delete(!read_only);
+}
+
+void Box_Data_List::set_open_button_title(const Glib::ustring& title)
+{
+  m_AddDel.set_open_button_title(title);
+}
+
+void Box_Data_List::set_primary_key_value_selected(const Gnome::Gda::Value& primary_key_value)
+{
+  Gtk::TreeModel::iterator iter = m_AddDel.get_row(primary_key_value);
+  if(iter)
+  {
+    m_AddDel.select_item(iter);
+  }
+}
+
+void Box_Data_List::get_record_counts(gulong& total, gulong& found) const
+{
+  Glib::RefPtr<Gtk::TreeModel> refModel = m_AddDel.get_model();
+  Glib::RefPtr<DbTreeModel> refModelDerived = Glib::RefPtr<DbTreeModel>::cast_dynamic(refModel);
+
+  refModelDerived->get_record_counts(total, found);
+}
+
+
+
 
