@@ -20,21 +20,23 @@
 
 #include "dialog_layout.h"
 //#include <libgnome/gnome-i18n.h>
-#include <libintl.h>
+#include <glibmm/i18n.h>
 
-Dialog_Layout::Dialog_Layout(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
+Dialog_Layout::Dialog_Layout(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade, bool with_table_title)
 : Gtk::Dialog(cobject),
   m_entry_table_title(0),
-  m_document(0),
   m_modified(false)
 {
   Gtk::Button* button = 0;
   refGlade->get_widget("button_close", button);
   button->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout::on_button_close) );
 
-  refGlade->get_widget("entry_table_title", m_entry_table_title);
-  m_entry_table_title->signal_changed().connect( sigc::mem_fun(*this, &Dialog_Layout::on_entry_table_title_changed) );
- 
+  if(with_table_title)
+  {
+    refGlade->get_widget("entry_table_title", m_entry_table_title);
+    m_entry_table_title->signal_changed().connect( sigc::mem_fun(*this, &Dialog_Layout::on_entry_table_title_changed) );
+  }
+
   show_all_children();
 }
 
@@ -42,16 +44,15 @@ Dialog_Layout::~Dialog_Layout()
 {
 }
 
-void Dialog_Layout::set_document(const Glib::ustring& layout, Document_Glom* document, const Glib::ustring& table_name, const type_vecFields& /* table_fields */)
+void Dialog_Layout::set_document(const Glib::ustring& layout, Document_Glom* /* document */, const Glib::ustring& table_name, const type_vecLayoutFields& /* table_fields */)
 {
   m_modified = false;
 
-  m_layout_name = layout;
-  
-  m_document = document;
+  m_layout_name = layout; 
+  //m_document = document;
   m_table_name = table_name;
-  
-  m_modified = false;  
+
+  m_modified = false;
 }
 
 void Dialog_Layout::move_treeview_selection_up(Gtk::TreeView* treeview, const Gtk::TreeModelColumn<guint>& sequence_column)
@@ -70,7 +71,7 @@ void Dialog_Layout::move_treeview_selection_up(Gtk::TreeView* treeview, const Gt
 
         Gtk::TreeModel::Row row = *iter;
         Gtk::TreeModel::Row rowBefore = **iterBefore;
-                                                     
+
         //Swap the sequence values, so that the one before will be after:
         guint tempBefore = rowBefore[sequence_column];
         guint tempRow = row[sequence_column];
@@ -80,7 +81,7 @@ void Dialog_Layout::move_treeview_selection_up(Gtk::TreeView* treeview, const Gt
         //Because the model is sorted, the visual order should now be swapped.
 
         m_modified = true;
-      }   
+      }
     }
 
   }
@@ -125,13 +126,13 @@ void Dialog_Layout::move_treeview_selection_down(Gtk::TreeView* treeview, const
 void Dialog_Layout::on_button_close()
 {
   save_to_document();
-  
+
   hide();
 }
 
 void Dialog_Layout::save_to_document()
 {
- 
+
 }
 
 
@@ -162,7 +163,7 @@ void Dialog_Layout::treeview_fill_sequences(const Glib::RefPtr<Gtk::TreeModel> m
 
     }
   }
-      
+
 }
 
 
@@ -180,4 +181,7 @@ void Dialog_Layout::enable_buttons()
 {
 }
 
-
+bool Dialog_Layout::get_modified() const
+{
+  return m_modified;
+}
