@@ -23,19 +23,17 @@
 
 #include <gtkmm/dialog.h>
 #include "../utility_widgets/dialog_properties.h"
-#include "../document/document_glom.h"
+#include <glom/libglom/document/document_glom.h>
 #include "../box_db.h"
 
 class Dialog_Layout :
   public Gtk::Dialog,
-  public View_Composite_Glom //Give it access to the document.
+  public Base_DB //Give it access to the document, and to the database utilities
 {
 public:
-  Dialog_Layout(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
+  Dialog_Layout(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade, bool with_table_title = true);
   virtual ~Dialog_Layout();
 
-  typedef std::vector< LayoutItem_Field > type_vecLayoutFields;
-
   /**
    * @param layout "list" or "details"
    * @param document The document, so that the dialog can load the previous layout, and save changes.
@@ -44,6 +42,8 @@ public:
    */
   virtual void set_document(const Glib::ustring& layout, Document_Glom* document, const Glib::ustring& table_name, const type_vecLayoutFields& table_fields);
 
+  virtual bool get_modified() const;
+
 protected:
 
   virtual void treeview_fill_sequences(const Glib::RefPtr<Gtk::TreeModel> model, const Gtk::TreeModelColumn<guint>& sequence_column);
