@@ -23,7 +23,7 @@
 
 #include <gtkmm.h>
 #include <libglademm.h>
-#include "../document/document_glom.h"
+#include <glom/libglom/document/document_glom.h>
 
 class Dialog_Design
 : public Gtk::Dialog,
@@ -33,7 +33,7 @@ public:
   Dialog_Design(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
   virtual ~Dialog_Design();
 
-  virtual void init_db_details(const Glib::ustring& strDatabaseName, const Glib::ustring& strTableName);
+  virtual bool init_db_details(const Glib::ustring& table_name);
 
 protected:
   virtual void on_button_close();
