@@ -23,10 +23,10 @@
 #include <glibmm/i18n.h>
 #include "../cellrendererlist.h"
 #include "db_treeviewcolumn_glom.h"
-#include "../../data_structure/glomconversions.h"
+#include <glom/libglom/data_structure/glomconversions.h>
 #include "../../dialog_invalid_data.h"
 #include "../../application.h"
-#include "../../utils.h"
+#include <glom/libglom/utils.h>
 #include "cellrenderer_button.h"
 //#include "../cellrendererlist.h"
 #include <iostream> //For debug output.
@@ -58,7 +58,13 @@ DbAddDelColumnInfo& DbAddDelColumnInfo::operator=(const DbAddDelColumnInfo& src)
 }
 
 DbAddDel::DbAddDel()
-: m_pMenuPopup(0),
+: m_column_is_sorted(false),
+  m_column_sorted_direction(false),
+  m_column_sorted(0),
+  m_pMenuPopup(0),
+  m_bAllowUserActions(true),
+  m_bPreventUserSignals(false),
+  m_bIgnoreTreeViewSignals(false),
   m_auto_add(true),
   m_allow_add(true),
   m_allow_delete(true),
@@ -68,12 +74,10 @@ DbAddDel::DbAddDel()
   m_treeviewcolumn_button(0)
 {
   set_prevent_user_signals();
-  set_ignore_treeview_signals();
+  set_ignore_treeview_signals(true);
 
   set_spacing(6);
 
-  m_bAllowUserActions = true;
-
   //Start with a useful default TreeModel:
   //set_columns_count(1);
   //construct_specified_columns();
@@ -82,6 +86,7 @@ DbAddDel::DbAddDel()
   m_ScrolledWindow.add(m_TreeView);
   m_ScrolledWindow.set_shadow_type(Gtk::SHADOW_IN);
 
+  m_TreeView.set_fixed_height_mode();
   m_TreeView.show();
   //m_TreeView.set_fixed_height_mode(); //This allows some optimizations.
   pack_start(m_ScrolledWindow);
@@ -569,7 +574,7 @@ void DbAddDel::construct_specified_columns()
   }
 
   //Create the model from the ColumnRecord:
-  m_refListStore = type_model_store::create(record, m_table_name, fields, column_index_key, m_allow_view, m_where_clause);
+  m_refListStore = type_model_store::create(record, m_found_set, fields, column_index_key, m_allow_view);
 
   m_TreeView.set_model(m_refListStore);
 
@@ -587,10 +592,15 @@ void DbAddDel::construct_specified_columns()
     GlomCellRenderer_Button* pCellButton = Gtk::manage(new GlomCellRenderer_Button());
     pCellButton->signal_clicked().connect(sigc::mem_fun(*this, &DbAddDel::on_cell_button_clicked));
 
-    const int viewcols_count = m_TreeView.append_column(Glib::ustring(), *pCellButton);
-    m_treeviewcolumn_button = m_TreeView.get_column(viewcols_count - 1);
 
-    m_treeviewcolumn_button->property_visible() = get_allow_view_details();
+    Gtk::TreeViewColumn* m_treeviewcolumn_button = Gtk::manage(new Gtk::TreeViewColumn());
+    m_treeviewcolumn_button->pack_start(*pCellButton);
+
+    m_treeviewcolumn_button->set_sizing(Gtk::TREE_VIEW_COLUMN_FIXED); //Need by fixed-height mode.
+    m_treeviewcolumn_button->set_fixed_width(25); //Otherwise it doesn't show up. TODO: Discover the width of the contents.
+    m_treeviewcolumn_button->property_visible() = true; //get_allow_view_details();
+
+    m_TreeView.append_column(*m_treeviewcolumn_button);
 
     ++view_column_index;
   }
@@ -762,7 +772,7 @@ bool DbAddDel::refresh_from_database()
   if(m_refListStore)
   {
     //Glib::RefPtr<Gtk::TreeModel> refNull;
-    bool result = m_refListStore->refresh_from_database(m_where_clause);
+    const bool result = m_refListStore->refresh_from_database(m_found_set);
     //m_TreeView.set_model(refNull); //TODO: This causes a g_warning(): gtk_tree_view_unref_tree_helper: assertion `node != NULL' failed
     if(m_TreeView.get_model())
       gtk_tree_view_set_model(m_TreeView.gobj(), 0); //This gives the same warning.
@@ -833,7 +843,7 @@ void DbAddDel::remove_all_columns()
 
 void DbAddDel::set_table_name(const Glib::ustring& table_name)
 {
-  m_table_name = table_name;
+  m_found_set.m_table_name = table_name;
 }
 
 guint DbAddDel::add_column(const sharedptr<const LayoutItem_Field>& field)
@@ -864,14 +874,14 @@ guint DbAddDel::add_column(const sharedptr<const LayoutItem_Field>& field)
   return m_ColumnTypes.size() - 1;
 }
 
-void DbAddDel::set_where_clause(const Glib::ustring& where_clause)
+void DbAddDel::set_found_set(const FoundSet& found_set)
 {
-  m_where_clause = where_clause;
+  m_found_set = found_set;
 }
 
-Glib::ustring DbAddDel::get_where_clause() const
+FoundSet DbAddDel::get_found_set() const
 {
-  return m_where_clause;
+  return m_found_set;
 }
 
 void DbAddDel::set_columns_ready()
@@ -886,13 +896,10 @@ DbAddDel::type_list_indexes DbAddDel::get_column_index(const sharedptr<const Lay
 
   type_list_indexes list_indexes;
 
-  const Glib::ustring field_name = layout_item->get_name();
-  const Glib::ustring relationship_name = layout_item->get_relationship_name();
-
   guint i = 0;
   for(type_ColumnTypes::const_iterator iter = m_ColumnTypes.begin(); iter != m_ColumnTypes.end(); ++iter)
   {
-    if( (iter->m_field->get_name() == field_name) && (iter->m_field->get_relationship_name() == relationship_name) )
+    if(iter->m_field->is_same_field(layout_item))
     {
       list_indexes.push_back(i);
     }
@@ -996,16 +1003,6 @@ void DbAddDel::finish_editing()
 //  set_ignore_treeview_signals(bIgnoreSheetSignals);
 }
 
-void DbAddDel::set_ignore_treeview_signals(bool bVal)
-{
-  m_bIgnoreSheetSignals = bVal;
-}
-
-bool DbAddDel::get_ignore_treeview_signals() const
-{
-  return m_bIgnoreSheetSignals;
-}
-
 /*
 void DbAddDel::reactivate()
 {
@@ -1032,6 +1029,16 @@ void DbAddDel::remove_item(const Gtk::TreeModel::iterator& iter)
     m_refListStore->erase(iter);
 }
 
+bool DbAddDel::get_ignore_treeview_signals() const
+{
+  return m_bIgnoreTreeViewSignals;
+}
+
+void DbAddDel::set_ignore_treeview_signals(bool ignore)
+{
+  m_bIgnoreTreeViewSignals = ignore;
+}
+
 DbAddDel::InnerIgnore::InnerIgnore(DbAddDel* pOuter)
 {
   m_pOuter = pOuter;
@@ -1041,7 +1048,7 @@ DbAddDel::InnerIgnore::InnerIgnore(DbAddDel* pOuter)
     m_bPreventUserSignals = m_pOuter->get_prevent_user_signals();
     m_pOuter->set_prevent_user_signals();
 
-    m_bIgnoreSheetSignals = m_pOuter->get_ignore_treeview_signals();
+    m_bIgnoreTreeViewSignals = m_pOuter->get_ignore_treeview_signals();
     m_pOuter->set_ignore_treeview_signals();
   }
 }
@@ -1053,7 +1060,7 @@ DbAddDel::InnerIgnore::~InnerIgnore()
   if(m_pOuter)
   {
     m_pOuter->set_prevent_user_signals(m_bPreventUserSignals);
-    m_pOuter->set_ignore_treeview_signals(m_bIgnoreSheetSignals);
+    m_pOuter->set_ignore_treeview_signals(m_bIgnoreTreeViewSignals);
   }
 
   m_pOuter = false;
@@ -1091,7 +1098,8 @@ void DbAddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, in
     Gnome::Gda::Value value_old;
     row.get_value(tree_model_column_index, value_old);
 
-    bool bValueNew = !(value_old.get_bool());
+    const bool bValueOld = (value_old.get_value_type() == Gnome::Gda::VALUE_TYPE_BOOLEAN) && value_old.get_bool();
+    bool bValueNew = !bValueOld;
     Gnome::Gda::Value value_new;
     value_new.set(bValueNew);
     //Store the user's new value in the model:
@@ -1386,7 +1394,9 @@ bool DbAddDel::on_treeview_column_drop(Gtk::TreeView* /* treeview */, Gtk::TreeV
 
 guint DbAddDel::treeview_append_column(const Glib::ustring& title, Gtk::CellRenderer& cellrenderer, int model_column_index)
 {
-  DbTreeViewColumnGlom* pViewColumn = Gtk::manage( new DbTreeViewColumnGlom(title, cellrenderer) );
+  DbTreeViewColumnGlom* pViewColumn = Gtk::manage( new DbTreeViewColumnGlom(GlomUtils::string_escape_underscores(title), cellrenderer) );
+  pViewColumn->set_sizing(Gtk::TREE_VIEW_COLUMN_FIXED); //Need by fixed-height mode.
+
   guint cols_count = m_TreeView.append_column(*pViewColumn);
 
   //Tell the TreeView how to render the Gnome::Gda::Values:
@@ -1408,12 +1418,45 @@ guint DbAddDel::treeview_append_column(const Glib::ustring& title, Gtk::CellRend
 
   //TODO pViewColumn->signal_button_press_event().connect( sigc::mem_fun(*this, &DbAddDel::on_treeview_columnheader_button_press_event) );
 
+  //Let the user click on the column header to sort.
+  pViewColumn->set_clickable();
+  pViewColumn->signal_clicked().connect(
+    sigc::bind( sigc::mem_fun(*this, &DbAddDel::on_treeview_column_clicked), model_column_index) );
+
   return cols_count;
 }
 
+void DbAddDel::on_treeview_column_clicked(int model_column_index)
+{
+  if(model_column_index >= (int)m_ColumnTypes.size())
+    return;
+
+  sharedptr<const LayoutItem_Field> layout_item = m_ColumnTypes[model_column_index].m_field;
+  if(layout_item && layout_item->get_name_not_empty())
+  {
+    bool ascending = true;
+    if(m_column_is_sorted && ((int)m_column_sorted == model_column_index))
+    {
+      //Reverse the existing direction:
+      ascending = !m_column_sorted_direction;
+    }
+
+    //Remember the user's chosen sort, so we can reverse it if he clicks again:
+    m_column_is_sorted = true;
+    m_column_sorted_direction = ascending;
+    m_column_sorted = model_column_index;
+
+    //Set the sort clause to be used by refresh_from_database():
+    m_found_set.m_sort_clause.clear();
+    m_found_set.m_sort_clause.push_back( type_pair_sort_field(layout_item, ascending) );
+  }
+
+  refresh_from_database();
+}
+
 void DbAddDel::on_treeview_columns_changed()
 {
-  if(!get_ignore_treeview_signals())
+  if(!m_bIgnoreTreeViewSignals)
   {
     //Get the new column order, and save it in m_vecColumnIDs:
     m_vecColumnIDs.clear();
@@ -1593,7 +1636,7 @@ void DbAddDel::treeviewcolumn_on_cell_data(Gtk::CellRenderer* renderer, const Gt
       {
         Gtk::CellRendererToggle* pDerived = dynamic_cast<Gtk::CellRendererToggle*>(renderer);
         if(pDerived)
-          pDerived->set_active( value.get_bool() ); 
+          pDerived->set_active( (value.get_value_type() == Gnome::Gda::VALUE_TYPE_BOOLEAN) && value.get_bool() ); 
 
         break;
       }
