@@ -22,10 +22,10 @@
 #define GLOM_DB_ADDDEL_H
 
 #include "gtkmm.h"
-#include "../../data_structure/layout/layoutitem_field.h"
+#include <glom/libglom/data_structure/layout/layoutitem_field.h>
 #include <libgdamm.h>
 #include "glom_db_treemodel.h"
-#include "../../document/document_glom.h"
+#include <glom/libglom/document/document_glom.h>
 
 #include <vector>
 #include <map>
@@ -39,7 +39,7 @@ public:
   DbAddDelColumnInfo(const DbAddDelColumnInfo& src);
   DbAddDelColumnInfo& operator=(const DbAddDelColumnInfo& src);
 
-  LayoutItem_Field m_field;
+  sharedptr<const LayoutItem_Field> m_field;
 
   typedef std::vector<Glib::ustring> type_vecStrings;
   type_vecStrings m_choices;
@@ -68,34 +68,34 @@ public:
   virtual void set_allow_add(bool val = true);
   virtual void set_allow_delete(bool val = true);
 
-  virtual Gtk::TreeModel::iterator add_item(const Gnome::Gda::Value& valKey); //Return index of new row.
+  //Gtk::TreeModel::iterator add_item(const Gnome::Gda::Value& valKey); //Return index of new row.
 
   /** Get an iterator to the blank row in which the user should add data for the new row.
    * You can then add the row to your underlying data store when some data has been filled, by handling signal_user_changed.
    */
-  virtual Gtk::TreeModel::iterator get_item_placeholder(); //Return index of the placeholder row.
+  Gtk::TreeModel::iterator get_item_placeholder(); //Return index of the placeholder row.
 
-  virtual void remove_item(const Gtk::TreeModel::iterator& iter);
+  void remove_item(const Gtk::TreeModel::iterator& iter);
 
-  virtual void remove_all();
+  void remove_all();
 
-  virtual Gnome::Gda::Value get_value(const Gtk::TreeModel::iterator& iter, const LayoutItem_Field& layout_item);
+  Gnome::Gda::Value get_value(const Gtk::TreeModel::iterator& iter, const sharedptr<const LayoutItem_Field>& layout_item);
 
   /** Get the row's hidden key
    */
-  virtual Gnome::Gda::Value get_value_key(const Gtk::TreeModel::iterator& iter);
+  Gnome::Gda::Value get_value_key(const Gtk::TreeModel::iterator& iter);
 
   /** Set the row's hidden key
    */
-  virtual void set_value_key(const Gtk::TreeModel::iterator& iter, const Gnome::Gda::Value& value);
+  void set_value_key(const Gtk::TreeModel::iterator& iter, const Gnome::Gda::Value& value);
 
   /** @param col A value returned from add_column().
    * @result The value on the selected row.
    */
-  virtual Gnome::Gda::Value get_value_selected(const LayoutItem_Field& layout_item);
-  virtual Gnome::Gda::Value get_value_key_selected();
+  Gnome::Gda::Value get_value_selected(const sharedptr<const LayoutItem_Field>& layout_item);
+  Gnome::Gda::Value get_value_key_selected();
 
-  virtual Gtk::TreeModel::iterator get_item_selected();
+  Gtk::TreeModel::iterator get_item_selected();
 
   /** 
    * @param iter The row to be selected. 
@@ -103,38 +103,38 @@ public:
    * @param start_editing Whether editing should start in the cell.
    * @result Whether the row was successfully selected.
    */
-  virtual bool select_item(const Gtk::TreeModel::iterator& iter, guint column, bool start_editing = false);  //bool indicates success.
-  virtual bool select_item(const Gtk::TreeModel::iterator& iter);
+  bool select_item(const Gtk::TreeModel::iterator& iter, guint column, bool start_editing = false);  //bool indicates success.
+  bool select_item(const Gtk::TreeModel::iterator& iter);
 
-  virtual guint get_count() const;
+  guint get_count() const;
 
   /** 
    * @param iter The row to be changed. 
    * @param layout_item Describes the column(s) whose values should be changed.
    * @param value The new value.
    */
-  virtual void set_value(const Gtk::TreeModel::iterator& iter, const LayoutItem_Field& layout_item, const Gnome::Gda::Value& value);
+  virtual void set_value(const Gtk::TreeModel::iterator& iter, const sharedptr<const LayoutItem_Field>& layout_item, const Gnome::Gda::Value& value);
 
   /** 
    * @param col A value returned from add_column().
    * @param value The new value.
    */
-  virtual void set_value_selected(const LayoutItem_Field& layout_item, const Gnome::Gda::Value& value);
+  virtual void set_value_selected(const sharedptr<const LayoutItem_Field>& layout_item, const Gnome::Gda::Value& value);
 
-  virtual bool get_is_first_row(const Gtk::TreeModel::iterator& iter) const;
-  virtual bool get_is_last_row(const Gtk::TreeModel::iterator& iter) const;
+  bool get_is_first_row(const Gtk::TreeModel::iterator& iter) const;
+  bool get_is_last_row(const Gtk::TreeModel::iterator& iter) const;
 
   /** @result Whether this is a blank row where date for a new row should be entered
    */
-  virtual bool get_is_placeholder_row(const Gtk::TreeModel::iterator& iter) const;
+  bool get_is_placeholder_row(const Gtk::TreeModel::iterator& iter) const;
 
-  virtual Field get_key_field() const;
-  virtual void set_key_field(const Field& field);
+  sharedptr<Field> get_key_field() const;
+  void set_key_field(const sharedptr<Field>& field);
 
-  virtual void set_table_name(const Glib::ustring& table_name);
+  void set_table_name(const Glib::ustring& table_name);
 
   ///For instance, if the user cannot view the table then don't try to get the records.
-  virtual void set_allow_view(bool val = true);
+  void set_allow_view(bool val = true);
 
   ///Whether each row should have a button, to request edit.
   virtual void set_allow_view_details(bool val = true);
@@ -143,42 +143,45 @@ public:
 
   /** @result The index of the new column.
    */
-  virtual guint add_column(const LayoutItem_Field& field);
+  guint add_column(const sharedptr<const LayoutItem_Field>& field);
 
   /// Specify which records to show:
-  virtual void set_where_clause(const Glib::ustring& where_clause);
+  void set_found_set(const FoundSet& found_set);
+
+  FoundSet get_found_set() const;
 
   /// Start using the added columns.
-  virtual void set_columns_ready();
+  void set_columns_ready();
 
-  virtual guint get_columns_count() const;
+  guint get_columns_count() const;
 
-  virtual LayoutItem_Field get_column_field(guint column_index) const;
+  sharedptr<const LayoutItem_Field> get_column_field(guint column_index) const;
 
   typedef DbAddDelColumnInfo::type_vecStrings type_vecStrings;
 
   /** Retrieves the column order, even after they have been reordered by the user.
    * @result a vector of column_id. These column_ids were provided in the call to add_column().
    */
-  virtual type_vecStrings get_columns_order() const;
+  type_vecStrings get_columns_order() const;
 
-  virtual void remove_all_columns();
+  void remove_all_columns();
   //virtual void set_columns_count(guint count);
   //virtual void set_column_title(guint col, const Glib::ustring& strText);
-  virtual void set_column_width(guint col, guint width);
+  void set_column_width(guint col, guint width);
 
   /// For popup cells.
-  virtual void set_column_choices(guint col, const type_vecStrings& vecStrings);
+  void set_column_choices(guint col, const type_vecStrings& vecStrings);
 
-  virtual void construct_specified_columns(); //Delay actual use of set_column_*() stuff until this method is called.
+  void construct_specified_columns(); //Delay actual use of set_column_*() stuff until this method is called.
 
-  virtual bool refresh_from_database();
+  bool refresh_from_database();
+  bool refresh_from_database_blank();
 
-  virtual void set_show_column_titles(bool bVal = true);
+  void set_show_column_titles(bool bVal = true);
 
-  virtual Gtk::TreeModel::iterator get_row(const Gnome::Gda::Value& key);
+  Gtk::TreeModel::iterator get_row(const Gnome::Gda::Value& key);
 
-  virtual void finish_editing(); //Closes active edit controls and commits the data to the cell.
+  void finish_editing(); //Closes active edit controls and commits the data to the cell.
   //virtual void reactivate(); //Sheet doesn't seem to update unless a cell is active.
   void set_prevent_user_signals(bool bVal = true);
 
@@ -186,12 +189,12 @@ public:
    * Use set_auto_add(false) if you want to provide default values for columns in the new row, or if you want to place the cursor in a different column.
    * If @a value is false then signal_user_requested_add will be emitted so that you can add the row explicitly.
    */
-  virtual void set_auto_add(bool value = true);
+  void set_auto_add(bool value = true);
 
   Glib::RefPtr<Gtk::TreeModel> get_model();
   Glib::RefPtr<const Gtk::TreeModel> get_model() const;
 
-  virtual void set_rules_hint(bool val = true);
+  void set_rules_hint(bool val = true);
 
   //Signals:
 
@@ -228,27 +231,20 @@ public:
 
   virtual Gtk::TreeModel::iterator get_last_row();
   virtual Gtk::TreeModel::iterator get_last_row() const;
-  
+
   virtual void set_open_button_title(const Glib::ustring& title);
 
 protected:
 
   typedef std::list<guint> type_list_indexes;
   ///Return the column indexes of any columns that display this field.
-  virtual type_list_indexes get_column_index(const LayoutItem_Field& layout_item) const;
-
-  /** Get an iterator to the blank row in which the user should add data for the new row.
-   * You can then add the row to your underlying data store when some data has been filled, by handling signal_user_changed.
-   */
-  virtual Gtk::TreeModel::iterator add_item_placeholder(); //Return index of new row.
+  virtual type_list_indexes get_column_index(const sharedptr<const LayoutItem_Field>& layout_item) const;
 
   virtual void setup_menu();
   virtual Gnome::Gda::Value treeview_get_key(const Gtk::TreeModel::iterator& row);
 
   ///Add a blank row, or return the existing blank row if there already is one.
-  virtual Gtk::TreeModel::iterator get_next_available_row_with_add_if_necessary();
-  virtual void add_blank();
-
+  //virtual Gtk::TreeModel::iterator get_next_available_row_with_add_if_necessary();
 
   //Signal handlers:
   void treeviewcolumn_on_cell_data(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter, int model_column_index);
@@ -269,14 +265,11 @@ protected:
   virtual void on_treeview_button_press_event(GdkEventButton* event);
 
   virtual bool on_treeview_columnheader_button_press_event(GdkEventButton* event);
-  virtual void on_cell_button_clicked(const Gtk::TreeModel::Path& path);  
+  virtual void on_treeview_column_clicked(int model_column_index);
+  virtual void on_cell_button_clicked(const Gtk::TreeModel::Path& path);
 
   bool get_prevent_user_signals() const;
 
-  //Sometimes the sheet sends signals when it shouldn't:
-  void set_ignore_treeview_signals(bool bVal = true);
-  bool get_ignore_treeview_signals() const;
-
   /** @param model_column_index A value returned from add_column().
    * @param view_column_index The index of the corresponding view column.
    */
@@ -289,8 +282,6 @@ protected:
 
   App_Glom* get_application();
 
-  static Glib::ustring string_escape_underscores(const Glib::ustring& text);
-
   typedef Gtk::VBox type_base;
 
   //Member widgets:
@@ -306,7 +297,12 @@ protected:
   //Columns, not including the hidden internal columns:
   typedef std::vector<DbAddDelColumnInfo> type_ColumnTypes;
   type_ColumnTypes m_ColumnTypes;
-  Glib::ustring m_table_name, m_where_clause;
+  FoundSet m_found_set; //table, where_clause, sort_clause.
+
+  bool m_column_is_sorted; //If empty, then m_column_sorted and m_column_sorted_direction should not be used.
+  bool m_column_sorted_direction; //true means ascending. 
+  guint m_column_sorted; //Previously-clicked (on the treeview header) column. Remember it so we can reverse the sort order on a second click.
+
   Glib::ustring m_open_button_title; //Allow us to change "Open" to "Select".
 
   Gtk::Menu* m_pMenuPopup;
@@ -317,7 +313,7 @@ protected:
   bool m_bAllowUserActions;
 
   bool m_bPreventUserSignals;
-  bool m_bIgnoreSheetSignals;
+  bool m_bIgnoreTreeViewSignals;
 
   type_vecStrings m_vecColumnIDs; //We give each ViewColumn a special ID, so we know where they are after a reorder.
 
@@ -325,7 +321,7 @@ protected:
   bool m_allow_add;
   bool m_allow_delete;
 
-  Field m_key_field;
+  sharedptr<Field> m_key_field;
 
   bool m_columns_ready;
   bool m_allow_view;
@@ -342,6 +338,9 @@ protected:
   type_signal_user_activated m_signal_user_activated;
   type_signal_user_reordered_columns m_signal_user_reordered_columns;
 
+  bool get_ignore_treeview_signals() const;
+  void set_ignore_treeview_signals(bool ignore = true);
+
   //An instance of InnerIgnore remembers the ignore settings,
   //then restores them when it goes out of scope and is destroyed.
   class InnerIgnore
@@ -352,7 +351,7 @@ protected:
 
   protected:
     DbAddDel* m_pOuter;
-    bool m_bPreventUserSignals, m_bIgnoreSheetSignals;
+    bool m_bPreventUserSignals, m_bIgnoreTreeViewSignals;
   };
 
 /*
