@@ -22,15 +22,15 @@
 #define GLOM_UTILITY_WIDGETS_ENTRY_GLOM_H
 
 #include <gtkmm.h>
-#include "../data_structure/field.h"
-#include "layoutwidgetbase.h"
+#include <glom/libglom/data_structure/field.h>
+#include "layoutwidgetfield.h"
 #include <libglademm.h>
 
 class App_Glom;
 
 class EntryGlom
 : public Gtk::Entry,
-  public LayoutWidgetBase
+  public LayoutWidgetField
 {
 public:
   explicit EntryGlom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
@@ -38,24 +38,21 @@ public:
   virtual ~EntryGlom();
 
   void set_glom_type(Field::glom_field_type glom_type);
- 
+
   //Override this so we can store the text to compare later.
   //This is not virtual, so you must not use it via Gtk::Entry.
   void set_text(const Glib::ustring& text); //override
 
   /** Set the text from a Gnome::Gda::Value.
    */
-  void set_value(const Gnome::Gda::Value& value);
+  virtual void set_value(const Gnome::Gda::Value& value);
 
-  Gnome::Gda::Value get_value() const;
+  virtual Gnome::Gda::Value get_value() const;
 
-  typedef sigc::signal<void> type_signal_edited;
-  type_signal_edited signal_edited();
-
-  typedef sigc::signal<void> type_signal_user_requested_layout;
-  type_signal_user_requested_layout signal_user_requested_layout(); 
+  virtual void set_read_only(bool read_only = true);
 
 protected:
+  void init();
 
   //Overrides of default signal handlers:
   virtual void on_changed(); //From Gtk::Entry.
@@ -65,16 +62,10 @@ protected:
 
   virtual void check_for_change();
 
-  virtual void setup_menu();
   virtual bool on_button_press_event(GdkEventButton *event); //override
-  virtual void on_menupopup_activate_layout();
-  virtual void on_menupopup_add_item(TreeStore_Layout::enumType item);
 
   virtual App_Glom* get_application();
 
-  type_signal_edited m_signal_edited;
-  type_signal_user_requested_layout m_signal_user_requested_layout;
-
   Glib::ustring m_old_text;
   Field::glom_field_type m_glom_type; //Store the type so we can validate the text accordingly.
 
